clear all;
clc;
close all;

%% concentric UCA settings
Nt_max = 600;
M = 20;
fc = 30e9;
c = 3e8;
lambda = c/fc;
d = lambda/2;
C = linspecer(6);

r_radius_max = Nt_max*d/2/pi; % half-wavelength spacing
D_fres = (2*r_radius_max)^(4/3)/2/(lambda)^(1/3);

r_radius_list = zeros(1, M);
Nt_list = zeros(1, M);
for i_m = 1:M
    r_radius_list(i_m) = r_radius_max/M*i_m;
    Nt_list(i_m) = Nt_max/M*i_m;
end
Nt_total = sum(Nt_list);

%% plot the beam gain in 3D -- theta=0
r_0 = 15;
theta_0 = 0;
phi_0 = 0;
[at, at_app, ~, ~] =...
    generate_con_UCA_3D(fc, Nt_list, r_radius_list, r_0, theta_0, phi_0);

num_r = 1000;
r_list = linspace(D_fres, 30, num_r); %D_fres
g_r = zeros(1, num_r);
g_r_app = zeros(1, num_r);
g_r_est = zeros(1, num_r);
g_r_theta0 = zeros(1, num_r);
for idx_r = 1:num_r
    r_iter = r_list(idx_r);
    [h0, h0_app,~] = generate_con_UCA_3D(fc, Nt_list, r_radius_list, r_iter, theta_0, phi_0);
    g_r(idx_r)=  abs(at*h0');
    g_r_app(idx_r)=  abs(at*h0_app');
    
    xi_1 = pi*r_radius_list.^2*sin(theta_0)^2/lambda/2*(1/(r_iter)-1/r_0);
    xi_2 = pi*r_radius_list.^2/lambda*(1/(r_iter)-1/r_0)*(1-1/2*sin(theta_0)^2);
    g_r_est(idx_r) = abs(sum(Nt_list.*besselj(0, xi_1).*exp(-1j*xi_2)))/sum(Nt_list);
    
    % estimation with close-form at theta=0
    zeta = pi*r_radius_max^2/lambda*(1/(r_iter)-1/r_0);
    g_r_theta0(idx_r) = abs(sin(zeta/2)/zeta*2);
end

figure;
hold on;
plot(r_list, g_r,'-','LineWidth',1.5,'color', C(1,:));
plot(r_list, g_r_theta0,'--','LineWidth',2.5,'color', C(2,:));
legend('Accurate Beamforming Gain', 'Estimated Beamforming Gain','Location','SouthEast');
xlabel('Distance of user r (m)');
ylabel('Normalized beamforming gain');
grid on;
box on;

%% plot the beam gain in 3D -- theta=pi/2
r_0 = 15;
theta_0 = pi/2;
phi_0 = 0;
[at, at_app, ~, ~] =...
    generate_con_UCA_3D(fc, Nt_list, r_radius_list, r_0, theta_0, phi_0);

num_r = 1000;
r_list = linspace(D_fres, 30, num_r); %D_fres
g_r = zeros(1, num_r);
g_r_app = zeros(1, num_r);
g_r_est = zeros(1, num_r);
for idx_r = 1:num_r
    r_iter = r_list(idx_r);
    [h0, h0_app,~] = generate_con_UCA_3D(fc, Nt_list, r_radius_list, r_iter, theta_0, phi_0);
    g_r(idx_r)=  abs(at*h0');
    g_r_app(idx_r)=  abs(at*h0_app');
    
    xi_1 = pi*r_radius_list.^2*sin(theta_0)^2/lambda/2*(1/(r_iter)-1/r_0);
    xi_2 = pi*r_radius_list.^2/lambda*(1/(r_iter)-1/r_0)*(1-1/2*sin(theta_0)^2);
    g_r_est(idx_r) = abs(sum(Nt_list.*besselj(0, xi_1).*exp(-1j*xi_2)))/sum(Nt_list);
end
% estimation with close-form at theta=pi/2
z0 = pi*r_radius_max^2/2/lambda*(1./(r_list)-1/r_0);
g_r_theta_pi_tmp = sqrt(besselj(0, z0).^2+besselj(1, z0).^2);

figure;
hold on;
plot(r_list, g_r,'-','LineWidth',1.5,'color', C(1,:));
plot(r_list, g_r_theta_pi_tmp,'--','LineWidth',2.5,'color', C(2,:));
legend('Accurate Beamforming Gain', 'Estimated Beamforming Gain','Location','SouthEast');
xlabel('Distance of user r (m)');
ylabel('Normalized beamforming gain');
grid on;
box on;

