clear all;
clc;
close all;

%% UCA settings
Nt = 600;
fc = 30e9;
c = 3e8;
lambda = c/fc;
d = lambda/2;
% xi = 1.5211; % delta = 0.5
xi = 0.9183; % delta = 0.8
eps = 1e-3;
C = linspecer(6);

r_radius = Nt*d/2/pi; % half-wavelength spacing
D = 2*(2*r_radius)^2/lambda;

%% plot the near-field region UCA
phi = 0;
thre_list = 0.8:0.05:0.8;
num_threshold = length(thre_list);
num_r = 2000;
num_theta = Nt/20;
theta_list = linspace(eps, pi-eps, num_theta);
for i_thre = 1:num_threshold
    threshold = thre_list(i_thre);
    
    r_list = zeros(num_theta,num_r);
    
    g = zeros(num_r, num_theta);
    r_center = 1000000;
    r0 = r_center;
    for idx_theta = 1:num_theta
        theta0 = theta_list(idx_theta);
        erd = pi*r_radius^2*sin(theta0)^2/2/lambda/xi;
        r_list(idx_theta,:) = linspace(erd/4, 60, num_r);
        
        [h0, h0_app] = generate_UCA_3D(fc, Nt, r_radius, r0, theta0, phi);
        for idx_r = 1:num_r
            [h_grid, h_grid_app] = generate_UCA_3D(fc, Nt, r_radius, r_list(idx_theta, idx_r), theta0, phi);
            g(idx_r, idx_theta)=  abs(h_grid*h0');
        end
    end

    [~, idx] = max(diag(1:num_r)*(g<threshold), [], 1);
    g_r_list=diag(r_list(1:num_theta,idx));
    g_r_est = pi*r_radius^2*sin(theta_list).^2/2/lambda/xi;
end

%% plot the near-field region UCCA
M = 20;
num_r = 2000;
r_radius_max = r_radius;
Nt_max = Nt;

r_radius_list = zeros(1, M);
Nt_list = zeros(1, M);
for i_m = 1:M
    r_radius_list(i_m) = r_radius_max/M*i_m;
    Nt_list(i_m) = Nt_max/M*i_m;
end
Nt_total = sum(Nt_list);

for i_thre = 1:num_threshold
    threshold = thre_list(i_thre);
    r_list = zeros(num_theta,num_r);
    g_ucca = zeros(num_r, num_theta);
    r_center = 1000000;
    r0 = r_center;
    for idx_theta = 1:num_theta
        theta0 = theta_list(idx_theta);
        r_list(idx_theta,:) = linspace(0, 60, num_r);
        [h0, h0_app,~] = generate_con_UCA_3D(fc, Nt_list, r_radius_list, r0, theta0, phi);
        for idx_r = 1:num_r
            [h_grid, h_grid_app,~] = generate_con_UCA_3D(fc, Nt_list, r_radius_list,  r_list(idx_theta, idx_r), theta0, phi);
            g_ucca(idx_r, idx_theta)=  abs(h_grid*h0');
        end
    end

    [~, idx] = max(diag(1:num_r)*(g_ucca<threshold), [], 1);
    g_r_list_ucca=diag(r_list(1:num_theta,idx));
end
figure;
hold on;
plot(theta_list, g_r_list,'-','LineWidth',1.5,'color', C(1,:));
plot(theta_list, g_r_est,'--','LineWidth',1.5,'color', C(2,:));
plot(theta_list, g_r_list_ucca,'-','LineWidth',1.5,'color', C(3,:));
xlabel('Elevation angle \theta');
ylabel('Effective Rayleigh distance (m)');
legend('Calculated ERD for UCA', 'Estimated ERD for UCA with (14)','Calculated ERD for UCCA');
grid on;
box on;
axis([0 pi 0 50]);