function [H, H_approx] = generate_UCA_3D(fc, Nt, r_radius, r_list, theta_list, phi_list)

c = 3e8;
lambda = c/fc;
[~, d_len] = size(r_list);

%% UCA settings
pp = 1:1:Nt;
phi_p = (pp-1)*2*pi/Nt;
[x_p, y_p] = pol2cart(phi_p, r_radius);

%% channel generation
H = zeros(d_len, Nt);
H_approx = zeros(d_len, Nt);
for i_d = 1:d_len
    theta0 = theta_list(i_d);
    r0 = r_list(i_d);
    r0_xy = r0*sin(theta0);
    r0_z = r0*cos(theta0);
    phi0 = phi_list(i_d);
    [x0, y0] = pol2cart(phi0, r0_xy);
    
    dis_xy = sqrt((x_p-x0).^2+(y_p-y0).^2);
    dis_xy_est = r0_xy-r_radius*cos(phi0-phi_p)+r_radius^2/2/r0_xy*(1-cos(phi0-phi_p).^2);
    err_xy = dis_xy-dis_xy_est;
    
    dis_sph = sqrt(r0^2+r_radius^2-2*r_radius*r0*sin(theta0)*cos(phi0-phi_p));
    dis_matrix = sqrt((x_p-x0).^2+(y_p-y0).^2+r0_z.^2);
    err_sph_matrix = dis_sph-dis_matrix;
    dis_acc = sqrt(r0^2+r_radius^2-2*r_radius*r0*sin(theta0)*cos(phi0-phi_p));
    err_acc = dis_matrix-dis_acc;
    dis_approx = r0-r_radius*sin(theta0)*cos(phi0-phi_p)+r_radius^2/2/r0*(1-sin(theta0)^2*cos(phi0-phi_p).^2);
    
    delta = (r_radius^2-2*r_radius*r0*sin(theta0)*cos(phi0-phi_p))/r0^2;
    dis_approx3 = r0*(1+delta/2-delta.^2/8+delta.^3/16);
    err = dis_matrix-dis_approx;
    at = exp(-1i*2*pi/lambda*dis_matrix)/sqrt(Nt);
    
    H(i_d, :) = at;
    H_approx(i_d, :) = exp(-1i*2*pi/lambda*dis_approx)/sqrt(Nt);
    
%     %% projection
%     at = exp(-1i*2*pi/lambda*dis_xy)/sqrt(Nt);
%     H(i_d, :) = at;
%     H_approx(i_d, :) = exp(-1i*2*pi/lambda*dis_xy_est)/sqrt(Nt);
end


end

